@@ -339,7 +339,7 @@ class Insight_Definition:
         return pd.DataFrame(data=result_set)
 
 
-    def add_chart(self, type="", query="", params=[], title = "Chart"):
+    def add_chart(self, type:str, query:str, params:list, title = "Chart"):
         position = (len(self.components))
         body = {"type": "chart", "position": position}
 
@@ -358,19 +358,37 @@ class Insight_Definition:
         r.raise_for_status()
         self.components = r.json()["components"]
 
-        if (type != "" and query != "" and params!=[]):
-            self.edit_chart(self.components[position]["id"], type, query, params, title)
-        else:
-            logging.info("One or more arguments between type, query and params are missing to update the chart")
+        self.edit_chart(self.components[position]["id"], type, query, params, title)
+
 
         return self.components[position]["id"]
 
 
-    def edit_chart(self, chart_id, chart_type:str, query, params, title = "Chart"):
+    def edit_chart(self, chart_id, chart_type:str, query:str, params, title = "Chart"):
+
+        url = self.smart_insight_url+"/definitions/"+self.definition_id+"/queries/"+query.lower()+"/columns"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        r = s.get(url=url, headers=headers)
+        r.raise_for_status()
+        columns = r.json()
+
+        upper_params = []
+        for param in params:
+            for column in columns:
+                if param in column["code"]:
+                    upper_params.append(column["code"])
 
         url = self.smart_insight_url + "/definitions/" + self.definition_id + "/charts/" + chart_id
 
-        upper_params = [p.upper() for p in params]
 
         body = {
             "chartType": chart_type.upper(),
@@ -454,13 +472,15 @@ class Insight_Definition:
         return data_id
 
 
-    def add_join_to_qc(self, qc_id, dataset_slug, fields, type:str):
+    def add_join_to_qc(self, qc_id, new_dataset_slug, new_fields, left_on:list, right_on:list, type:str ,filters:list=[], sorted_by= None, limit=50, offset=0, pivot=[]):
 
-        new_qc = self.create_qc(dataset_slug, fields)
+        new_qc = self.create_qc(new_dataset_slug, new_fields, filters, sorted_by, limit= limit, pivot=pivot, offset=offset)
 
         url = self.smart_insight_url+"/composed_queries/{}".format(qc_id)
 
-        body = {
+        qc = self.get_qc(qc_id)
+        qc = qc["settings"]["qcSettings"]
+        qc["joinedSmartQuery"]= {
             "smartQueryId": new_qc["id"]
         }
 
@@ -473,27 +493,35 @@ class Insight_Definition:
             "Content-Type": "application/json",
             "Authorization": "Bearer" + " " + self._token
         }
-        r = s.put(url=url, json=body, headers=headers)
+        r = s.put(url=url, json=qc, headers=headers)
         r.raise_for_status()
         response = r.json()
 
-        qc = self.get_qc(qc_id)
+        print("RESPONSE 1 {}".format(response))
+        right_fields = []
+        left_fields = []
+        for field in new_qc["fields"]:
+            if field["column"] in right_on:
+                right_fields.append(field)
+        for field in qc["fields"]:
+            if field["column"] in left_on:
+                left_fields.append(field)
 
-        right_fields = qc["qcSettings"]["fields"]
-        left_fields = new_qc[fields]
         joinable_fields = response["joinedSmartQuery"]["joinableFields"]
 
-        body = {
-            "smartQueryId": new_qc["id"],
-            "joinType": type.upper(),
-            "leftFields": left_fields,
-            "rightFields": right_fields,
-            "dataset": {
-                "id": new_qc["datasets"][0]["id"],
-                "slug": dataset_slug
-            },
-            "joinableFields": joinable_fields
+
+
+        qc["joinedSmartQuery"]["smartQueryId"] = new_qc["id"]
+        qc["joinedSmartQuery"]["joinType"] = type.upper()
+        qc["joinedSmartQuery"]["leftFields"] = left_fields
+        qc["joinedSmartQuery"]["rightFields"] = right_fields
+        qc["joinedSmartQuery"]["joinableFields"] = joinable_fields
+        qc["joinedSmartQuery"]["dataset"] = {
+            "id": new_qc["datasets"][0]["dataset"],
+            "slug": new_dataset_slug
         }
+
+
         s = requests.Session()
         s.keep_alive = False
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
@@ -503,7 +531,7 @@ class Insight_Definition:
             "Content-Type": "application/json",
             "Authorization": "Bearer" + " " + self._token
         }
-        r = s.put(url=url, json=body, headers=headers)
+        r = s.put(url=url, json=qc, headers=headers)
         r.raise_for_status()
         response = r.json()
 
