@@ -521,6 +521,185 @@ class Query:
         spell_str += "}"
         return spell_str
 
+    def _where_spell_out_untyped(self, where):
+        conditions = []
+
+        for where_condition in where:
+            if isinstance(where_condition, Condition):
+                vars = []
+                if where_condition.value is not None:
+                    for var in where_condition.value:
+                        vars.append(str(var))
+                    formatted_value = "( " + ", ".join(vars) + " )"
+                else:
+                    formatted_value = None
+
+                if isinstance(where_condition.operator, str):
+                    operator = where_condition.operator
+                elif isinstance(where_condition.operator, SQLOperator) or isinstance(where_condition.operator,
+                                                                                     TimeOperator):
+                    operator = where_condition.operator.name
+                    if operator == "GOE":
+                        operator = ">="
+                    elif operator == "LOE":
+                        operator = "<="
+                    elif operator == "EQ":
+                        operator = "=="
+                    elif operator == "GT":
+                        operator = ">"
+                    elif operator == "LT":
+                        operator = "<"
+                else:
+                    operator = None
+
+                if ("measure" in where_condition.field.internalDataType.lower() and "measure" not in where_condition.field.sourceDataType.lower()) or \
+                        ("dimension" in where_condition.field.internalDataType.lower() and "dimension" not in where_condition.field.sourceDataType.lower()):
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type ) AS Alias
+                    formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ") " + operator + " " + formatted_value
+                elif "timedimension" in where_condition.field.internalDataType.lower() and "timedimension" not in where_condition.field.sourceDataType.lower():
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type, date_format YYYY-MM-DD ) AS Alias
+                    if where_condition.field.sourceFormat is not None:
+                        formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format " + where_condition.field.sourceFormat + ") " + operator + " " + formatted_value
+                    else:
+                        formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ", to " + where_condition.field.internalDataType + ", format YYYY-MM-DD" + ") " + operator + " " + formatted_value
+                else:
+                    # else
+                    formatted_cond = where_condition.field.column + " (" + where_condition.field.sourceDataType + ") " + operator + " " + formatted_value
+
+                conditions.append(formatted_cond)
+            elif isinstance(where_condition, CompositeCondition):
+                other_conds = self._where_spell_out(where_condition.conditions)
+                if where_condition.operator.name == "AND":
+                    conditions = other_conds
+                elif where_condition.operator.name == "OR":
+                    conditions.append(" OR ".join(other_conds))
+        return conditions
+
+    def spell_out_untyped(self):
+        spell_str = '# {"fields":['
+
+        fields = []
+        for field in self.fields:
+            # if there is sql expression it always wins
+            if field.sqlExpression is not None:
+                if field.alias is None:
+                    if field.aggregation is not None and isinstance(field.aggregation, str):
+                        string = '"' + field.aggregation + " " + field.sqlExpression + '"'
+                    elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                        string = '"' + field.aggregation.name + " " + field.sqlExpression + '"'
+                    else:
+                        string = '"' + field.sqlExpression + '"'
+                else:
+                    if field.aggregation is not None and isinstance(field.aggregation, str):
+                        string = '"' + field.aggregation + " " + field.sqlExpression + " AS " + field.alias + '"'
+                    elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                        string = '"' + field.aggregation.name + " " + field.sqlExpression + " AS " + field.alias + '"'
+                    else:
+                        string = '"' + field.sqlExpression + " AS " + field.alias + '"'
+            else:
+                # otherwise, check if internal and original data types are not consistent each other
+                if ("measure" in field.internalDataType.lower() and "measure" not in field.sourceDataType.lower()) or \
+                        ("dimension" in field.internalDataType.lower() and "dimension" not in field.sourceDataType.lower()):
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type ) AS Alias
+                    if field.alias is None:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ")" + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ")" + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ")" + '"'
+                    else:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ") AS " + field.alias + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ") AS " + field.alias + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ") AS " + field.alias + '"'
+                elif "timedimension" in field.internalDataType.lower() and "timedimension" not in field.sourceDataType.lower():
+                    # if internal and original data types are not consistent each other use this pattern
+                    # aggregation column (original data type, to internal data type, date_format YYYY-MM-DD ) AS Alias
+                    if field.sourceFormat is not None:
+                        if field.alias is None:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ")" + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ")" + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ")" + '"'
+                        else:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ") AS " + field.alias + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ") AS " + field.alias + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format " + field.sourceFormat + ") AS " + field.alias + '"'
+                    else:
+                        if field.alias is None:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ")" + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ")" + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ")" + '"'
+                        else:
+                            if field.aggregation is not None and isinstance(field.aggregation, str):
+                                string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ") AS " + field.alias + '"'
+                            elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                                string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ") AS " + field.alias + '"'
+                            else:
+                                string = '"' + field.column + " (" + field.sourceDataType + ", to " + field.internalDataType + ", format YYYY-MM-DD" + ") AS " + field.alias + '"'
+                else:
+                    # else
+                    if field.alias is None:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ")" + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ")" + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ")" + '"'
+                    else:
+                        if field.aggregation is not None and isinstance(field.aggregation, str):
+                            string = '"' + field.aggregation + " " + field.column + " (" + field.sourceDataType + ") AS " + field.alias + '"'
+                        elif field.aggregation is not None and isinstance(field.aggregation, SQLFunction):
+                            string = '"' + field.aggregation.name + " " + field.column + " (" + field.sourceDataType + ") AS " + field.alias + '"'
+                        else:
+                            string = '"' + field.column + " (" + field.sourceDataType + ") AS " + field.alias + '"'
+            fields.append(string)
+        spell_str += ", ".join(fields) + "]"
+
+        if self.datasets is not None and self.datasets:
+            spell_str += ', "from":["{table}"]'
+
+        if self.where is not None and self.where:
+            spell_str += ', "where":['
+            condition_str = self._where_spell_out_untyped(self.where)
+            condition_str = ['"' + cond + '"' for cond in condition_str]
+            spell_str += ", ".join(condition_str) + "]"
+
+        if self.orderBy is not None and self.orderBy:
+            spell_str += ', "order_by":['
+            orderby_vec = []
+            for clause in self.orderBy:
+                if isinstance(clause.order, SQLSorting):
+                    orderby_vec.append('"' + clause.field + " " + clause.order.name + '"')
+                else:
+                    orderby_vec.append('"' + clause.field + " " + clause.order + '"')
+            spell_str += ", ".join(orderby_vec) + "]"
+
+        if self.limit is not None:
+            spell_str += ', "limit": ' + str(self.limit)
+
+        if self.offset is not None:
+            spell_str += ', "offset": ' + str(self.offset)
+        else:
+            spell_str += ', "offset": 0'
+
+        spell_str += "}"
+        return spell_str
+
 
 @dataclass
 class SmartQuery:
@@ -535,6 +714,13 @@ class SmartQuery:
             spell_array.append(spell)
         return spell_array
 
+    def spell_out_untyped(self):
+        spell_array = []
+        for query in self.queries:
+            spell = query.spell_out_untyped()
+            spell_array.append(spell)
+        return spell_array
+
     @staticmethod
     def _get_tokens():
         f = open(os.path.join(os.path.dirname(__file__), "askdata_config", "compression_tokens.json"))
